{% extends "!layout.html" %}

{% block fonts %}
    <style>
        body {
            font-family: 'Liberation Sans', -apple-system, BlinkMacSystemFont, 'Segoe UI', Roboto, Oxygen, Ubuntu, Cantarell, 'Open Sans', 'Helvetica Neue', sans-serif;
        }
        .header-style {
            font-family: 'Liberation Sans', Cambria, Cochin, Georgia, Times, 'Times New Roman', serif;
        }
    </style>
{% endblock %}

{# when reading on a mobile you do want prev/next buttons at the ened as well {%
block relbar2 %}{% endblock %} #}
