/* extends insipid.css_t to have smaller prev/next links
*/

.relbar > a {
    padding: 0px;
    padding-top: 12px;
    color: black;
}

.relbar .title {
    font-size: 80%;  // Luc 20201025 1.4em;
}

.menuselection { font-weight: bold; font-family: monospace; }
.guilabel { font-weight: bold; font-family: monospace; }
